"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.initAdapterManager = initAdapterManager;
exports.isUsingAdapter = isUsingAdapter;
exports.setWebpackAssets = setWebpackAssets;
var _reporter = _interopRequireDefault(require("gatsby-cli/lib/reporter"));
var _pageHtml = require("gatsby-core-utils/page-html");
var _pageData = require("gatsby-core-utils/page-data");
var _path = require("path");
var _glob = require("glob");
var _gatsbyTelemetry = _interopRequireDefault(require("gatsby-telemetry"));
var _redux = require("../../redux");
var _pageMode = require("../page-mode");
var _staticQueryUtils = require("../static-query-utils");
var _init = require("./init");
var _enginesHelpers = require("../engines-helpers");
var _constants = require("./constants");
var _createHeaders = require("./create-headers");
var _types = require("../../redux/types");
var _rankRoute = require("../rank-route");
var _getRoutePath = require("./get-route-path");
function noOpAdapterManager() {
  return {
    restoreCache: () => {},
    storeCache: () => {},
    adapt: () => {}
  };
}
let usingAdapter = false;
function isUsingAdapter() {
  return usingAdapter;
}
async function initAdapterManager() {
  let adapter;
  const config = _redux.store.getState().config;
  const {
    adapter: adapterFromGatsbyConfig
  } = config;

  // If the user specified an adapter inside their gatsby-config, use that instead of trying to figure out an adapter for the current environment
  if (adapterFromGatsbyConfig) {
    adapter = adapterFromGatsbyConfig;
    _reporter.default.verbose(`Using adapter ${adapter.name} from gatsby-config`);
  } else {
    const adapterInit = await (0, _init.getAdapterInit)();

    // If we don't have adapter, use no-op adapter manager
    if (!adapterInit) {
      _gatsbyTelemetry.default.trackFeatureIsUsed(`adapter:no-op`);
      usingAdapter = false;
      return noOpAdapterManager();
    }
    adapter = adapterInit();
  }
  usingAdapter = true;
  _reporter.default.info(`Using ${adapter.name} adapter`);
  _gatsbyTelemetry.default.trackFeatureIsUsed(`adapter:${adapter.name}`);
  const directoriesToCache = [`.cache`, `public`];
  return {
    restoreCache: async () => {
      // TODO: Remove before final merge
      _reporter.default.info(`[Adapters] restoreCache()`);
      if (!adapter.cache) {
        return;
      }
      const result = await adapter.cache.restore({
        directories: directoriesToCache,
        reporter: _reporter.default
      });
      if (result === false) {
        // if adapter reports `false`, we can skip trying to re-hydrate state
        return;
      }
      const cachedState = (0, _redux.readState)();

      // readState() returns empty object if there is no cached state or it's corrupted etc
      // so we want to avoid dispatching RESTORE_CACHE action in that case
      if (Object.keys(cachedState).length > 0) {
        _redux.store.dispatch({
          type: `RESTORE_CACHE`,
          payload: cachedState
        });
      }
    },
    storeCache: async () => {
      // TODO: Remove before final merge
      _reporter.default.info(`[Adapters] storeCache()`);
      if (!adapter.cache) {
        return;
      }
      await adapter.cache.store({
        directories: directoriesToCache,
        reporter: _reporter.default
      });
    },
    adapt: async () => {
      // TODO: Remove before final merge
      _reporter.default.info(`[Adapters] adapt()`);
      if (!adapter.adapt) {
        return;
      }
      let _routesManifest = undefined;
      let _functionsManifest = undefined;
      const adaptContext = {
        get routesManifest() {
          if (!_routesManifest) {
            _routesManifest = getRoutesManifest();
          }
          return _routesManifest;
        },
        get functionsManifest() {
          if (!_functionsManifest) {
            _functionsManifest = getFunctionsManifest();
          }
          return _functionsManifest;
        },
        reporter: _reporter.default
      };
      await adapter.adapt(adaptContext);
    }
  };
}
let webpackAssets;
function setWebpackAssets(assets) {
  webpackAssets = assets;
}
function getRoutesManifest() {
  const routes = [];
  const state = _redux.store.getState();
  const createHeaders = (0, _createHeaders.createHeadersMatcher)(state.config.headers);
  const fileAssets = new Set((0, _glob.sync)(`**/**`, {
    cwd: _path.posix.join(process.cwd(), `public`),
    nodir: true,
    dot: true
  }));

  // TODO: This could be a "addSortedRoute" function that would add route to the list in sorted order. TBD if necessary performance-wise
  function addRoute(route) {
    if (!route.path.startsWith(`/`)) {
      route.path = `/${route.path}`;
    }
    if (route.type !== `function`) {
      route.headers = createHeaders(route.path, route.headers);
    }
    ;
    route.score = (0, _rankRoute.rankRoute)(route.path);
    routes.push(route);
  }
  function addStaticRoute({
    path,
    pathToFillInPublicDir,
    headers
  }) {
    addRoute({
      path,
      type: `static`,
      filePath: _path.posix.join(`public`, pathToFillInPublicDir),
      headers
    });
    if (fileAssets.has(pathToFillInPublicDir)) {
      fileAssets.delete(pathToFillInPublicDir);
    } else {
      _reporter.default.warn(`[Adapters] Tried to remove "${pathToFillInPublicDir}" from assets but it wasn't there`);
    }
  }

  // routes - pages - static (SSG) or function (DSG/SSR)
  for (const page of state.pages.values()) {
    const htmlRoutePath = (0, _getRoutePath.getRoutePathFromPage)(page);
    const pageDataRoutePath = (0, _pageData.generatePageDataPath)(``, htmlRoutePath);
    const pageMode = (0, _pageMode.getPageMode)(page);
    if (pageMode === `SSG`) {
      const htmlFilePath = (0, _pageHtml.generateHtmlPath)(``, page.path);
      const pageDataFilePath = (0, _pageData.generatePageDataPath)(``, page.path);
      addStaticRoute({
        path: htmlRoutePath,
        pathToFillInPublicDir: htmlFilePath,
        headers: _constants.STATIC_PAGE_HEADERS
      });
      addStaticRoute({
        path: pageDataRoutePath,
        pathToFillInPublicDir: pageDataFilePath,
        headers: _constants.STATIC_PAGE_HEADERS
      });
    } else {
      const commonFields = {
        type: `function`,
        functionId: `ssr-engine`
      };
      if (pageMode === `DSG`) {
        commonFields.cache = true;
      }
      addRoute({
        path: htmlRoutePath,
        ...commonFields
      });
      addRoute({
        path: pageDataRoutePath,
        ...commonFields
      });
    }
  }

  // static query json assets
  for (const staticQueryComponent of state.staticQueryComponents.values()) {
    const staticQueryResultPath = (0, _staticQueryUtils.getStaticQueryPath)(staticQueryComponent.hash);
    addStaticRoute({
      path: staticQueryResultPath,
      pathToFillInPublicDir: staticQueryResultPath,
      headers: _constants.STATIC_PAGE_HEADERS
    });
  }

  // app-data.json
  {
    const appDataFilePath = _path.posix.join(`page-data`, `app-data.json`);
    addStaticRoute({
      path: appDataFilePath,
      pathToFillInPublicDir: appDataFilePath,
      headers: _constants.STATIC_PAGE_HEADERS
    });
  }

  // webpack assets
  if (!webpackAssets) {
    // TODO: Make this a structured error
    throw new Error(`[Adapters] webpackAssets is not defined`);
  }
  for (const asset of webpackAssets) {
    addStaticRoute({
      path: asset,
      pathToFillInPublicDir: asset,
      headers: _constants.WEBPACK_ASSET_HEADERS
    });
  }

  // TODO: slices

  // redirect routes
  for (const redirect of state.redirects.values()) {
    var _redirect$statusCode;
    addRoute({
      path: redirect.fromPath,
      type: `redirect`,
      toPath: redirect.toPath,
      status: (_redirect$statusCode = redirect.statusCode) !== null && _redirect$statusCode !== void 0 ? _redirect$statusCode : redirect.isPermanent ? _types.HTTP_STATUS_CODE.MOVED_PERMANENTLY_301 : _types.HTTP_STATUS_CODE.FOUND_302,
      ignoreCase: redirect.ignoreCase,
      headers: _constants.REDIRECT_HEADERS
    });
  }

  // function routes
  for (const functionInfo of state.functions.values()) {
    addRoute({
      path: `/api/${(0, _getRoutePath.getRoutePathFromFunction)(functionInfo)}`,
      type: `function`,
      functionId: functionInfo.functionId
    });
  }

  // TODO: Remove before final merge
  console.log(`[Adapters] unmanaged (or not yet handled) assets`, fileAssets);
  for (const fileAsset of fileAssets) {
    addStaticRoute({
      path: fileAsset,
      pathToFillInPublicDir: fileAsset,
      headers: _constants.ASSET_HEADERS
    });
  }
  return routes.sort((a, b) => {
    // The higher the score, the higher the specificity of our path
    const order = b.score - a.score;
    if (order !== 0) {
      return order;
    }

    // if specificity is the same we do lexigraphic comparison of path to ensure
    // deterministic order regardless of order pages where created
    return a.path.localeCompare(b.path);
  })
  // The score should be internal only, so we remove it from the final manifest
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  .map(({
    score,
    ...rest
  }) => {
    return {
      ...rest
    };
  });
}
function getFunctionsManifest() {
  const functions = [];
  for (const functionInfo of _redux.store.getState().functions.values()) {
    const pathToEntryPoint = _path.posix.join(`.cache`, `functions`, functionInfo.relativeCompiledFilePath);
    functions.push({
      functionId: functionInfo.functionId,
      pathToEntryPoint,
      requiredFiles: [pathToEntryPoint]
    });
  }
  if ((0, _enginesHelpers.shouldGenerateEngines)()) {
    function getFilesFrom(dir) {
      return (0, _glob.sync)(`**/**`, {
        cwd: _path.posix.join(process.cwd(), dir),
        nodir: true,
        dot: true
      }).map(file => _path.posix.join(dir, file));
    }
    functions.push({
      functionId: `ssr-engine`,
      pathToEntryPoint: _path.posix.join(`.cache`, `page-ssr`, `lambda.js`),
      requiredFiles: [...getFilesFrom(_path.posix.join(`.cache`, `data`, `datastore`)), ...getFilesFrom(_path.posix.join(`.cache`, `page-ssr`)), ...getFilesFrom(_path.posix.join(`.cache`, `query-engine`))]
    });
  }
  return functions;
}
//# sourceMappingURL=manager.js.map